#! /usr/bin/env python
'''
Module Name:    Utils.py

Description:    Contains helper functions 

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2015 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None

Restrictions:    None

Last Modified: 30 September 2013

'''
import os
import sys
import subprocess
import time

#importing helper modules 
from monitorupgrade import TLog as TLOG

#Globals
FILE_FOUND = 1
FILE_NOT_FOUND = 0
EXIT_SUCCESS = 0
EXIT_FAILURE = 1 
UNKNOWN = -1
NO_MONITORS_ATTACHED = 0


#Error Codes
ERROR_ATMEL_UPDATE_FAILED = 2
ERROR_UPDATE_FAILED = 3
ERROR_CODE_DEVICE_NOT_EXIST = 4
ERROR_CODE_ATMEL_DEVICE_NOT_EXIST = 5
ERROR_CODE_DWU_DEVICE_NO_UPDATE = 6
ERROR_CODE_DWU_DEVICE_NOT_CONNECTED = 9
ERROR_CODE_DWU_DEVICE_NOT_FOUND = 10
ERROR_CODE_OPERATION_FAILED = 11
ERROR_CODE_AIP_SEGFAULT = 12
ERROR_DWU_VIDEO_UPDATE_FAILED = 20
ERROR_CODE_INVALID_PARAM = 21
ERROR_RETRY = 99
ERROR_SEGFAULT = 139

#Error Strings
ERROR_STR_DEVICE_NOT_EXIST = 'Device not exist'
ERROR_STR_OPERATION_FAILED = 'Operation failed'
ERROR_STR_INVALID_PARAM = 'Invalid Parameter'
ERROR_STR_Runtime = 'Runtime error'

MUS_LOG_FILE_DIR = '/var/log/toshiba/mus'

#sampo stuffs here
SAMPO_UTIL = '/opt/toshiba/mus/op_utils/sampo/4820MonUtil'
RECOVERY_FILE_SUSE = '/etc/init.d/after.local'
RECOVERY_STR = '4820MonUtil'

#ELO Stuffs here    
ELO_DOWNLOAD = '/opt/toshiba/mus/op_utils/elo/EloDownload.sh'
ELO_CONFIG = '/opt/toshiba/mus/op_utils/elo/EloConfig.sh'
ELO_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/elo/recovery.txt'
ATMEL_XCFG_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/mxtapp/xcfg_recovery.txt'
ATMEL_FW_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/mxtapp/fw_recovery.txt'
AIP_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/aipfwupdate/recovery.txt'
BOOT_COUNTER ='/tmp/reboot_count'

#Atmel
MXT_APP = '/usr/bin/mxt-app'
BOOT_LOADER_STR='03EB:211D'
ATMEL_SERIAL_LEN = 20

#
# Each line of lstmon2 is abstracted into a dict with these keys. We have constants for the keys
# as the dict can be shared across modules. Each dict is added to a list and returned by getMonList..
#
MON_HUB_ID = 'hub_id'
MON_ENTITY = 'entity'
MON_VIDPID = 'vid_pid'
MON_BCDDEVICE = 'bcdDevice' 
MON_USB_DEVID = 'usb_dev_id'
MON_HWID = 'hwid'
MON_SERIAL = 'serial'

#
# Code Flags
#
# Verbose debug
ADBG=True

#==========================================================================================================
'''
doesFileExist - Returns True if the file exist, False otherwise
'''
#==========================================================================================================

def doesFileExist(filename):
    
    if os.path.exists(filename) and os.path.isfile(filename) and os.access(filename, os.R_OK):
        #print filename + ' exists and is readable'
        TLOG.logger.info('doesFileExist()-Found! :'+ filename)
        return True
    else:
        #print filename  + ' does not exist'
        TLOG.logger.info('doesFileExist()-Not Found! :'+ filename)
        #raise IOError
        return False
#==========================================================================================================
'''
runShell - run any shell command
'''
#==========================================================================================================
def runShell(cmd,cwd_= None):
    
    #TLOG.logger.info(cmd)    
    process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,cwd=cwd_)
    while True:
        line = process.stdout.readline()
        #print line
        if not line:
            break
        TLOG.logger.info(line)    
    
    process.wait()
    return process.returncode



#==========================================================================================================
'''
runSubprocess - This will call the subprocess module to run the command in the cmd string
'''
#==========================================================================================================
def runSubprocess(cmd,proc,cwd_=None,time_out=60):
    
    TLOG.logger.info( str(cmd) )
    #std_err = None
    #try:
    process = subprocess.Popen(cmd,shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE, cwd = cwd_ )        
    
    while True:
        line = process.stdout.readline()
        #print line
        if not line:
            break
        line = line.strip('\n')
        TLOG.logger.info('        Subprocess[' + proc + '] OUTPUT-->' + ''  +  line )
        msg = ''    
        if (proc == 'SampoVideoFlashTask_4820_21x51x' or proc == 'SampoVideoFlashTask_4820_2Lx5Lx'):
            if "DATA" in line:
                idx = line.find(">")
                for c in line[idx + 1:len(line)]:
                    if c == '<':
                        break
                    msg += c
                print msg + '\r'  
    
    process.wait()
    ret = process.returncode
    
    if ADBG: print '-----------------------------------------------------------------------------'
    if ADBG: print 'runSubprocess returns:', ret, str(hex(ret))
    if ADBG: print '-----------------------------------------------------------------------------'

    try:
        if ret == 139 and proc == 'ELO_RECOVER':               # TODO: ELO Recovery Success but with Segmentation Fault
            if ADBG: print "ret == 139 and proc == 'ELO_RECOVER' ... ELO Recovery Success but with Segmentation Fault???"
            return EXIT_SUCCESS
        
        #
        # There are libmaxtouch MXT_ERROR_* codes in libmaxtouch.h 
        # but not all get through and are emitted on the mxt-app command line. 
        # I found:
        # MXT_FIRMWARE_UPDATE_NOT_REQUIRED = 21,     /*!< Device firmware already required version */
        #
        #
        elif (ret == 20 ) :                                                                     # TODO: Error code after Successfully loading Atmel Configuration - Need to verify with Atmel
            if ADBG: print "ret == 20 .. Error code after Successfully loading Atmel Configuration - Need to verify with Atmel???"
            return EXIT_SUCCESS
        
        elif (ret == ERROR_CODE_DWU_DEVICE_NO_UPDATE ) :
                #print 'DWU: No Update is Needed'
                if ADBG: print "ret == ERROR_CODE_DWU_DEVICE_NO_UPDATE .. DWU: No Update is Needed???"
                return EXIT_SUCCESS
        
        elif ret != EXIT_SUCCESS:
                  
            #TLOG.logger.error('return code:' + str(ret) )
            
            #when the monitor is unplugged during Firmware update.
            if ( ret == ERROR_CODE_DEVICE_NOT_EXIST  or ret == ERROR_UPDATE_FAILED  or ret == ERROR_CODE_DWU_DEVICE_NOT_CONNECTED
                 or ret == ERROR_CODE_DWU_DEVICE_NOT_FOUND or ret == ERROR_ATMEL_UPDATE_FAILED or ret == ERROR_DWU_VIDEO_UPDATE_FAILED ) :  
                
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_Runtime
                raise RuntimeError('Device not exist')
            
            elif (ret == ERROR_CODE_DWU_DEVICE_NOT_FOUND or ret == ERROR_CODE_ATMEL_DEVICE_NOT_EXIST) :  
                return ERROR_RETRY
               
            elif (ret == ERROR_CODE_OPERATION_FAILED ) :
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_OPERATION_FAILED
            
            elif (ret == ERROR_CODE_INVALID_PARAM ) :
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_INVALID_PARAM
            
            #FLIWA - Added for segfault error with libusb when there are 4820 and 6149 monitors attached
            elif (ret == ERROR_SEGFAULT or ret == ERROR_CODE_AIP_SEGFAULT ) :
                TLOG.logger.info('ERROR: SEGFAULT')
                return ERROR_CODE_AIP_SEGFAULT
                
            elif (proc != 'ELOCONFIGUTILITY'):
                print '[TaskMgr::runSubprocess()] Error:' + str(ret)
                #raise IOError(ret)
                raise RuntimeError(ret)
            #else:
                #raise IOError(ret)
        else:
            
            return EXIT_SUCCESS
    
    except (ValueError):
        
        extype, value = sys.exc_info()[:2]
        print 'Value Error'     
        print >> sys.stderr, "Invalid argument in Popen"
        TLOG.logger.error("%s (%s)" % (extype, value) )
        # return 2 for "bad command line option"
        sys.exit(2)
    
    except (IOError):
        
        extype, value = sys.exc_info()[:2]
        print 'IO Error'     
        #print >> sys.stderr, "Error in process: %s" % proc 
        TLOG.logger.error("TaskMgr::runSubprocess()-[Error in process: %s] %s (s%)" % ( proc,repr(extype), repr(value) ) )
 
        cleanUp(False)
        
        return EXIT_FAILURE
        
    except (RuntimeError):      
       
        value = sys.exc_info()[:2]
        #print value
        print 'Runtime Error'
        #print >> sys.stderr, "TaskMgr::runSubprocess() - Error in process: %s" % proc 
        TLOG.logger.error("TaskMgr::runSubprocess()-[RuntimeError in process: [%s] [%s]" % ( proc, repr(value) ) )
        
        #if ( proc == 'SampoVideoFlashTask_4820_2Lx5Lx' or proc == 'SampoVideoFlashTask_4820_21x51x' ):
            #cleanUp(True)       # Sets the recovery value to True
        #elif (proc == 'ELODOWNLOAD-4820_2Lx5Lx' or proc == 'ELODOWNLOAD-4820_21x51x') :
            #cleanUp(False)
        return EXIT_FAILURE          
            
    except (OSError):
        
        extype, value = sys.exc_info()[:2]
        print 'OS Error'     
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, value)
        TLOG.logger.error("%s (%s)" % (extype, value) )
        # return 2 for "bad command line option"
        sys.exit(2)
    
    except:
        # all other exceptions get caught here
        extype, value = sys.exc_info()[:2]
        print 'CatchAll Error'
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, repr(value) )
        TLOG.logger.error("%s (%s)" % (extype,  repr(value) ) )
        sys.exit(1)

# find all 614x monitors in bootloader mode
def find614Xin211Dmode():
        
        mon_list=[]
        query_cmd = MXT_APP  + ' -q > /tmp/count ' 
        process = subprocess.Popen(query_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait()                  
    
        #mon_list = []
        f = open('/tmp/count', 'r+')
        lines = f.readlines()
        f.close() 
        
        process = subprocess.Popen('rm /tmp/count ', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait() 
        
        for line in lines:
            line = line.strip('\n')
            if BOOT_LOADER_STR in line:
                usb_id, chip, vid_pid = line.split(' ')
                print 'Device in Boot loader mode:'+ usb_id, chip, vid_pid
                mon_list.append(usb_id)
        
        return mon_list


def find614X():
        
        mon_list=[]
        query_cmd = MXT_APP  + ' -q > /tmp/count ' 
        process = subprocess.Popen(query_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait()                  
    
        #mon_list = []
        f = open('/tmp/count', 'r+')
        lines = f.readlines()
        f.close() 
        
        process = subprocess.Popen('rm /tmp/count ', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait() 
        
        for line in lines:
            line = line.strip('\n')
            usb_id, chip, vid_pid = line.split(' ')
            print 'Device recovered:'+ usb_id, chip, vid_pid
            mon_list.append(usb_id)
        
        return mon_list           
 
     
#==========================================================================================================
# cleanUp() -  delete firmware files from the fw_temp directory which MUS checks
#           -  delete updatask.conf configuration file
#==========================================================================================================        
def cleanUp(in_recovery):
        
        TLOG.logger.info('cleanUp()------------------START--------------- in recovery='+str(in_recovery))    
        
        if (not in_recovery):
            
            if ( doesFileExist(ELO_RECOVERY_FILE) ) :
                cmd1 = 'rm' +' ' + ELO_RECOVERY_FILE
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
                if ADBG: print cmd1
                rem_proc = runShell(cmd1)
            
            '''
            if ( doesFileExist(ATMEL_FW_RECOVERY_FILE) ) :
                cmd1 = 'rm' + ' ' + ATMEL_FW_RECOVERY_FILE 
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
                if ADBG: print cmd1
                rem_proc = runShell(cmd1)
         
            if ( doesFileExist(ATMEL_XCFG_RECOVERY_FILE) ) :
                cmd1 = 'rm' + ' ' + ATMEL_XCFG_RECOVERY_FILE 
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
                if ADBG: print cmd1
                rem_proc = runShell(cmd1)
         '''
                
            if ( doesFileExist(AIP_RECOVERY_FILE) ) :
                cmd1 = 'rm' +  ' ' + AIP_RECOVERY_FILE
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
                if ADBG: print cmd1
                rem_proc = runShell(cmd1)            
           
        else:
            #:TODO Waaahhhhhh!
            if ( doesFileExist(ELO_RECOVERY_FILE) or doesFileExist(AIP_RECOVERY_FILE) ) :
                cmd1 = 'rm' + ' ' + ELO_RECOVERY_FILE + ' '  + AIP_RECOVERY_FILE
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
            
                if ADBG: print cmd1
                rem_proc = runShell(cmd1)
                if rem_proc == 0:
                    runShell('reboot')
                    
        TLOG.logger.info('cleanUp()-------------------END----------------\n')     
        
        
def doEloRecover(fw_path):
        
        TLOG.logger.info('doEloRecover() -----------------START---------------')
        eloRecoverCmd = 'sh ' +  ELO_DOWNLOAD + '  -select=fix  -firmware_file=' + str(fw_path)
        print eloRecoverCmd
        if doesFileExist(ELO_DOWNLOAD) != True:
            print 'EloAnakFlashTask::run()-No ELO Download script found!'
            TLOG.logger.error('Elo4820_21x51xFlashTask::run()-No ELO Download script found!')
            raise IOError('No ELO Download script found!')
        else:
            ret = runSubprocess(eloRecoverCmd,'ELO_RECOVER')
             
        if ret != EXIT_SUCCESS:
            return ret #return immediately
        else:
            print 'doEloRecover() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
            TLOG.logger.info('doEloRecover() -----------------END-----------------\n')
            return ret
        

def doAipRecover(fw_path,fw_name):
        
        TLOG.logger.info('doAipRecover() -----------------START---------------')
        
        fw_path = fw_path.strip('\'')
        fw_name = fw_name.strip('\'')
        aipRecoverCmd = 'aipfwupdate'  +  ' -F  ' + str(fw_name) + ' -f'
        
        print aipRecoverCmd
        TLOG.logger.info('doAipRecover():' +  aipRecoverCmd)
        ret = runSubprocess(aipRecoverCmd,'AIP_RECOVER',str(fw_path))
        
        if (ret == ERROR_SEGFAULT or ret == ERROR_CODE_AIP_SEGFAULT or 
            ret == -11 or ret == -6) :                                  #TODO: ret == -11 or ret == -6  These error always occurs in recovery, need to investigate
            
            count = 0
            #Try to do internal recovery up to number of retries
            while(ret != 0 and count != 3):    
                txt = 'doAipRecover-----------------RETRY#'+ str(count+1) + '-----------------\n'
                TLOG.logger.info(txt)
                ret = runSubprocess(aipRecoverCmd,'VideoFwFlashTask_6149_DWU',str(fw_path))
                count += 1
                
            if ret != 0:
                    print "===================================================WARNING!======================================================"
                    
                    print "\nYou may have a attached a combination of 6149 and 4820 type monitors while performing 6149 Monitor Video Firmware.\n"
                    print "Please unplug the attached 4820 Monitors to proceed firmware update recovery of 6149 Monitors.\n"
                    
                    TLOG.logger.info('ERROR; SEGFAULT During Recovery of Video Firmware Update, please unplug then replug your monitor.\n')
                    TLOG.logger.info('You may have a attached a combination of 6149 and 4820 type monitors while performing 6149 Monitor Video Firmware.\n')
                    TLOG.logger.info('Please unplugged the attached 4820 Monitors to proceed firmware update recovery of 6149 Monitors.\n')
                    
                    msg = '\"Press any key to continue...\"'
                    runShell('read -p  ' + msg + ' -n1 -s')
           
                    
        if ret != EXIT_SUCCESS:
            return ret #return immediately
        else:
            print 'doAipRecover() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
            TLOG.logger.info('doAipRecover() -----------------END-----------------\n')
            return ret
        

#      
# MOVING RECOVERY LOGIC TO AtmelTouchFwFlashRecovery CLASS
#
def doAtmelRecover_old(fw_path,conf):
        
        TLOG.logger.info('doAtmelRecover() -----------------START---------------')
        
        fw_file_path = fw_path.rstrip('\'')
        cfg_path = conf.rstrip('\'')
     
        
        mon_list = find614Xin211Dmode()
        #flash firmware
        if (len(mon_list) != 0 ):
                for mon in mon_list:
                    
                    Atmel_Recover_cmd = MXT_APP + ' -d ' + str(mon)  + ' --flash ' + fw_file_path + ' --verbose=2'            
                    print "[doAtmelRecover] - This will take some time,Please wait ..."
                    
                    ret = runSubprocess(Atmel_Recover_cmd,'ATMEL_RECOVER')
                    if ret != 0:
                        return ret
                        
        else:
                    Atmel_Recover_cmd = MXT_APP + ' --flash ' + fw_file_path
                    ret = runSubprocess(Atmel_Recover_cmd,'ATMEL_RECOVER')
        
        mon_list = find614X()
        #Load config
        if (len(mon_list) != 0 ):
                for mon in mon_list:
                    print "[doAtmelRecover] - Loading Configuration now..."
                    mxt_app_load_cmd = MXT_APP +  ' -d'+ str(mon) + ' --load ' + cfg_path                                            
                    #mxt_app_load_cmd = MXT_APP + ' --load ' + cfg_path 
                    ret = runSubprocess(mxt_app_load_cmd,'MXT-APP')
                    # Ah but then?
                    mxt_app_bkup_cmd = MXT_APP + ' -W -T6 -r1 55 '
                    ret = runSubprocess(mxt_app_bkup_cmd,'MXT-APP')
                    if ret != 0:
                        return ret
                        
        
        if ret != EXIT_SUCCESS:
            return ret #return immediately
        else:
            print 'doAtmelRecover() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
            TLOG.logger.info('doAtmelRecover() -----------------END-----------------\n')
            return ret

#
# For DAWU Touch we have implemented a reboot count file in the associated task base directory
#
REBOOT_FILE = "reboot"

def getRebootCount(reboot_path):
    '''
    Return an integer stored in the 'reboot' file in the task base directory associated with a task
    
    return None if any error occurs
    return 0 if 'reboot' file does not exist in a valid 'reboot_path'
    return 0 if a line with the reboot count <n> is not in the file
    return the stored count as an integer if found
    '''
    if reboot_path is None:
        return None   
    if not os.path.exists(reboot_path):
        return None
    #
    reboot_file = os.path.join(reboot_path,REBOOT_FILE)
    if not os.path.exists(reboot_file):
        return 0
    #
    count = 0
    try:
        f = open(reboot_file,'r+')
        line = f.readline()
        if ADBG: print line
        if line:
            cnt_str = line.strip('\n')
            count = int(cnt_str)
        f.close()
    except IOError as e:
        print "I/O error({0}): {1}".format(e.errno, e.strerror)
        return None
    except ValueError:
        print "Could not convert data to an integer."
        return None
    except:
        print "Unexpected error:", sys.exc_info()[0]
        return None
    return count


def setRebootCount(reboot_path, new_count):
    '''
    Store integer new_count in the 'reboot' file in the task base directory associated with a task.
    If 'reboot' file does not yet exist create new 'reboot' file
    Store as a line: <n>
    
    return False if any error occurs
    '''
    if reboot_path is None:
        return False
    if not os.path.exists(reboot_path):
        return False
    if not isinstance(new_count, int):
        return False
    #
    reboot_file = os.path.join(reboot_path,REBOOT_FILE)
    try:
        #
        # Open/create 'reboot' file
        #
        f=open(reboot_file,'a+')
        line = f.readline()
        if ADBG: print line
        f.seek(0)
        f.truncate()
        new_reboot_line = str(new_count)+'\n'
        f.write(new_reboot_line)
        f.flush()
        f.close()
    except IOError as e:
        print "I/O error({0}): {1}".format(e.errno, e.strerror)
        return False
    except ValueError:
        print "Could not convert data to an integer."
        return False
    except:
        print "Unexpected error:", sys.exc_info()[0]
        return False
    return True



'''
def updateBootCount():
    
        current_count=0
        if doesFileExist(BOOT_COUNTER) != True:
            f = open(BOOT_COUNTER,'w+')
            f.write('1')
            f.close()
        else:
            f = open(BOOT_COUNTER,'r+')
            lines = f.readlines()
            for line in lines:
                if not line:
                    break
                if line.isdigit() == True:
                    current_count = int(line.strip('\n'))
            
            #updating the counter
            f.seek(0)           # beginning of the file
            f.truncate()        # empty the file
            
            current_count += 1  # increment by 1
            
            f.write( str(current_count) )
            f.close()

def getBootCount():
        current_count=0
        if doesFileExist(BOOT_COUNTER) == True:
            f = open(BOOT_COUNTER,'r+')
            lines = f.readlines()
            for line in lines:
                if not line:
                    break
                if line.isdigit() == True:
                    current_count = int(line.strip('\n'))
                    
        return current_count
'''
#gf->
#==========================================================================================================
'''
Wrapper around lstmon2
'''
#==========================================================================================================
def getMonList():
    '''
        getMonitorList2 - function to get the list of all attached monitors and its details --Video/Touch FW, etc
        
        lstmon2 Output Format:     Hn [MFT] [VID:PID] [FW-Level or Levels] [bus/dev] [Hardware ID] [(USB) Serial NUmber]
        
           Where:
        
             Hn is Hub number.
                   Example: H0, H1
                   Lines with the same hub number are under the same hub - i.e. on the same monitor
        
              MFT is a 3 Letter ID for the USB Device
                   M: Manufacturer - Implies Flash/Config Interface spec:
                        T = Toshiba
                        S = SAMPO
                        E = ELO
                        3 = 3M (not implemented yet)
                        A = Atmel/Atmel
                   F: Family of Monitor
                        A = Anak
                        Y = Yami
                        K = Yami or Anak in don't care situations.
                        D = Dawu
                        R = Darth (not implemented yet)
                        I = Ichigan
                   T: Type of Device
                        V = Video Controller
                        T = Touch Controller
                        D = Atmel controller in DFU Mode 03eb:211d
        
            VID:PID is Vendor id & Product id.
        
            FW is device firmware level(s)
                1. One level string of 4 digits
                    or
                2. A comma delimited string of 4 digit levels
                    This used for Dawu MCU - which has 3 firmware levels:
                    MCU,Capsense,Scalar.and a config level at index 1.
                    //
                    Now it is also used for Dawu Touch in the form:
                    'FFFF,XX'
                    where FFFF i sthe four digit hex firmware level
                    and XX is the two digit hex XCFG level
        
            Atmel HW DESIGNATOR ID
                _Only for Atmel_ but we add to all the output strings. If it is not available the
                string "NOHWID" will be placed at this position in the output string as a
                placeholder. This makes parsing the strings more reliable.
        
                Note:
                No HWID is available if an Atmel chip is in DFU mode
        
                Atmel
                --------
                Distinguishes Atmel, ELO and other Atmel controller/touch-panel versions from each other
                Of the form:
                    F156-1664S-nnn
        
                    where nnn
                        Dawu    Atmel        001
                        Dawu    ELO        011
                        Dawu Qisda        021
        
                        Ichigan Atmel    001
                        Ichigan ELO        011
                        Ichigan Qisda    021
        
                        Daza    ELO        031
                        Daza Qisda        041
        
            USB Serial#
                Can distinguish between instances of a monitor chip
                For Atmel it is the string found at T38 Offset 18 as well as the USB Serial
        
                Note: This program (lstmon2) encloses the serial number in single quotes as
                the brain-dead Atmel USB serial number is taken from the T-60 and has
                trailing spaces to pad it a fixed length. E.g. 'SERIAL              '
    '''
    
    # monlist is an array of controller chip descriptor lines. Each descriptor is a dict.
    monlist = []
    
    # Notes: Liwa had a shell >> append here - and resetting self.mon_list was commented out??? See unused fn above
    process = subprocess.Popen('lstmon2  >  /tmp/monlist.txt', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    process.wait()          
        
    f = open('/tmp/monlist.txt', 'r+')
    rawlines = f.readlines()
    f.close()
       
    process = subprocess.Popen('rm /tmp/monlist.txt ', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    process.wait()
        
    for rawline in rawlines:
        line = rawline.replace('\n','')    # replace all \n with empty string
        if ADBG: print line
        parts = line.split("'")                         # "   '   "
        if ADBG: print '->  ', parts, ' ( len: ', len(parts),')'
        if len(parts) < 2:
            TLOG.logger.error('Utils::getMonList - Not enough main parts in the lstmon2 string: ' + parts)
            raise IndexError('Not enough parts!')
    
        main_part = parts[0].split()
        if len(main_part) < 6:
            TLOG.logger.error('Utils::getMonList - Not enough items in first part of parsed lstmon2 string: ' + main_part)
            raise IndexError('Not enough items in main_part!')
            
        # Get and parse out the first part of the entity string - up to but not including the 'serial number'
        if ADBG: print '   ->  ', main_part
        hub_id = main_part[0].strip()                      # use as key for temp object
        entity = main_part[1].strip()                        # ADT/SAV/... 
        vid_pid = main_part[2].strip()                      # Controller VID/PID
        bcdDevice = main_part[3].strip()                # Firmware version or comma delimited versions
        usb_dev_id = main_part[4].strip()              # Linux kernel USB Host-Device string
        hwid = main_part[5].strip()                           # Currently only for Atmel. Others have a dummy 'NOHWID" placeholder
            
        if ADBG: print '   ->  hub_id:', hub_id, 'entity:', entity, 'vid_pid:', vid_pid, 'bcdDevice:',bcdDevice, 'usb_dev_id:', usb_dev_id, 'hwid:', hwid
            
        # Strip down the serial and then build back up to fixed length (20) again iff it is Atmel
        serial_part = parts[1].strip()
        if (main_part[1]=='ADT' or main_part[1]=='AIT'):
            ln = len(serial_part)
            p = (ATMEL_SERIAL_LEN - ln) if (ln < ATMEL_SERIAL_LEN) else 0
            for i in range(p):
                serial_part += ' '
        serial = serial_part
        if ADBG: print '   ->  ', serial, ' ( len: ', len(serial),')'
        
        mon = {}
        mon[MON_HUB_ID] = hub_id
        mon[MON_ENTITY] = entity
        mon[MON_VIDPID] = vid_pid
        mon[MON_BCDDEVICE] = bcdDevice 
        mon[MON_USB_DEVID] = usb_dev_id
        mon[MON_HWID] = hwid
        mon[MON_SERIAL] = serial
        monlist.append(mon)
    #---ENDFOR---#
        
    return monlist


def getUsbDevidForSerial(match_serial):
    monlist = getMonList()
    for mon in monlist:
        if mon[MON_SERIAL]==match_serial:
            return mon[MON_USB_DEVID]
    return 'UNKNOWN'

